/*
** Module   : tty.clipboard.js
** Abstract : a simple text clipboard implementation.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140701 First version.
**                  For selection draw drag the mouse.
**                  For copy use CTRL-C key after selection.
**                  For paste use CTRL-V key.
**                  Use ESC to clear selection. 
*/

tty.clipboard = (function() 
{
   /* exports object */
   var my = {};
   
   /* CTRL-V pressed*/
   my.ctrlv = false;
   
   /* have selection */
   var selection = false;
   
   /* stroke rectangle selection */
   var stroke = true;
   
   /* selection color*/
   var color = 'blue';

   /* font metrics */
   var charWidth;
   var charHeight;
      
   /* canvas objects */
   var canvas;
   var ctx;
   
   /* selected cells */
   var cells = {};
   
   /* selection rectangle */
   var rect = {};
   
   /* drag flag */
   var drag = false;
      
   /* input element used on copy/paste operations */
   var inputText;
   
   /**
    * Init module.
    * 
    * @param {type} cfg configurartion object
    */
   my.init = function(cfg)
   {
      // input element used on copy/paste operations.
      inputText = document.getElementById(cfg.clipboard.input);
      
      charWidth = tty.screen.charWidth;
      charHeight = tty.screen.charHeight;

      canvas = document.getElementById(cfg.clipboard.id);
      canvas.width = tty.screen.canvasWidth;
      canvas.height = tty.screen.canvasHeight;
      
      stroke = cfg.clipboard.stroke;
      
      ctx = canvas.getContext('2d');
      ctx.fillStyle = "white"; 
      ctx.globalAlpha = 0;
      ctx.clearRect(0, 0, canvas.width, canvas.height);
            
      canvas.addEventListener('mousedown', mouseDown, false);
      canvas.addEventListener('mouseup', mouseUp, false);
      canvas.addEventListener('mousemove', mouseMove, false);

      //firefox
      if (!window.Clipboard)
      {
         inputText.focus();
      }
   };
   
   /**
    * Mouse down event handler.
    * 
    * @param {type} e Event object
    * @returns {undefined}
    */
   function mouseDown(e) 
   {
      if (e.button === 0)
      {
         // start drag
         var x = (e.pageX - this.offsetLeft) | 0;
         var y = (e.pageY - this.offsetTop) | 0;
         cells.startX = (x / charWidth) | 0;
         cells.startY = (y / charHeight) | 0;         
         rect.startX = cells.startX * charWidth;
         rect.startY = cells.startY * charHeight;
         drag = true;
      }
   }

   /**
    * Mouse up event handler. Draw selection rectangle.
    * 
    * @param {type} e Event object
    * @returns {undefined}
    */
   function mouseUp(e) 
   {
     if (drag && e.button === 0)
     {
        drag = false;
     }
   }

   /**
    * Mouse move event handler.
    * 
    * @param {type} e Event object
    * @returns {undefined}
    */
   function mouseMove(e) 
   {
      if (drag && e.button === 0) 
      {
         var x = (e.pageX - this.offsetLeft | 0);
         var y = (e.pageY - this.offsetTop | 0);
         cells.startX1 = (x / charWidth) | 0;
         cells.startY1 = (y / charHeight) | 0;

         rect.w = (cells.startX1 - cells.startX + 1) * charWidth;
         rect.h = (cells.startY1 - cells.startY + 1) * charHeight;

         ctx.clearRect(0, 0, canvas.width, canvas.height);         

         if (stroke)
         {
            ctx.globalAlpha = 1.0;
            ctx.beginPath();
            ctx.strokeStyle = color;      
            ctx.strokeRect(rect.startX, rect.startY, rect.w, rect.h);
            ctx.stroke();            
         }
         else
         {
            ctx.globalAlpha = 0.2;
            ctx.fillStyle = color;
            ctx.fillRect(rect.startX, rect.startY, rect.w, rect.h);
         }
         
         selection = true;
      }
   }      

   /**
    * Clipboard paste handler.
    * 
    * @param {type} e event object
    * @returns {undefined}
    */
   window.addEventListener("paste", function pasteHandler(e)
   {
      var text = "";
 
      if (window.clipboardData) 
      {   
         text = window.clipboardData.getData("Text");
      }
      else if (e.clipboardData) 
      {
         text = e.clipboardData.getData("text/plain");
      }

      if (isEmpty(text))
      {
         // if empty string and CTRL-V was pressed       
         if (my.ctrlv)
         {
            // send CTRL-V
            var msg = new Uint8Array(3);
            msg[0] = 0x03;
            msg[1] = 0;
            msg[2] = 22;
            tty.socket.send(msg);
         }
      }
      else      
         paste(text);
      
      my.ctrlv = false;
      e.preventDefault();
   });
   
   /**
    * Send paste text down to server to be injected
    * into the keyboard queue. 
    * 
    * @param {type} text   text to send.
    * @returns {undefined}
    */
   function paste(text)
   {
      if (text.length > 0)
      {
         var msg = new Uint8Array(text.length * 2 + 1);
         msg[0] = 0x03;

         for (var i = 0, j = 1;i < text.length;i++, j += 2)
         {
            var c = text.charCodeAt(i);
            msg[j + 1] = c & 0xff;
            c = c >> 8;
            msg[j] = c & 0xff;
         }
         
         tty.socket.send(msg); 
      }                     
   }

   /**
    * Clipboard copy handler.
    * 
    * @param {type} e event object
    * @returns {undefined}
    */
   window.addEventListener("copy", function copyHandler(e)
   {
      // value to copy
      var text = inputText.value;
      
      // copy to clipboard
      if (window.clipboardData) 
      {
         window.clipboardData.clearData();
         window.clipboardData.setData("Text", text);
      }
      else if (e.clipboardData) 
      {
         e.clipboardData.clearData();
         e.clipboardData.setData("text/plain", text);
      }      
      
      // clear selection
      my.clearSelection();
      e.preventDefault();
   });
   
   /**
    * Prepare for copy operation.
    * 
    * @returns {undefined}
    */
   my.prepareCopy = function()
   {
      if (selection)
      {
         // rows
         var begRow = Math.min(cells.startY, cells.startY1);
         var endRow = Math.max(cells.startY, cells.startY1);
         // cols
         var begCol = Math.min(cells.startX, cells.startX1);
         var endCol = Math.max(cells.startX, cells.startX1);
         // get selected text
         var text = tty.screen.copy(begRow, endRow, begCol, endCol);               
         // copy selection to clipboard
         inputText.value = text;
         inputText.focus();
         inputText.select();
      }
   };
      
   /**
    * Prepare for paste operation.
    * 
    * @returns {undefined}
    */
   my.preparePaste = function()
   {
      inputText.select();
   };

   /**
    * Check if a string is empty.
    * 
    * @param {str} input string 
    * @returns true if empty false otherwise
    */
   function isEmpty(str) 
   {
      return (!str || 0 === str.length);
   }   
   
   /**
    * Clear selection on screen.
    * 
    * @returns {undefined}
    */
   my.clearSelection = function()
   {
      selection = false;
      ctx.clearRect(0, 0, canvas.width, canvas.height);
   };
   
   /**
    * Check if have selection.
    * 
    * @returns {boolean} true if have a selection false if not.
    */
   my.haveSelection = function()
   {
      return selection;
   };

   /* exports */
   return my;   
}());
