/*
** Module   : tty.keyboard.js
** Abstract : keystrokes management.
**
** Copyright (c) 2014, Golden Code Development Corporation.
** ALL RIGHTS RESERVED. Use is subject to license terms.
**
**         Golden Code Development Corporation
**                 CONFIDENTIAL
**
** -#- -I- --Date-- ----------------------Description--------------------------
** 001 MAG 20140110 First version.
** 002 MAG 20140228 Add javadoc.
** 003 MAG 20140402 Implements interactive process.
** 004 MAG 20140507 Fix DEL key code for VT100 mode.
** 005 MAG 20140701 Use CTRL-C and CTRL-V to copy/paste.
*/

tty.keyboard = (function()
{
   /* exports object */
   var my = {};    
   
   /* mode vt100=true p2j=false */
   my.vt100 = false;
   
   /* track escape status */
   var escape = false;                     
   
   /* track alt char status */
   var altChar = false;

   /**
    * Initialize module.
    */
   my.init = function()
   {      
      /* init mapping */   
      tty.keymap.init();
   };
                    
   /**
    * Key press event.
    * 
    * @param {event} evt  event object.
    */
   my.onkeypress = function(evt)
   {
      var key = evt.charCode;

       // Prevent Firefox keypress on CTRL_C and CTRL-V
      if (tty.isFirefox && evt.ctrlKey)
      {
         if (key === 99 || key === 118)
         {
            return;
         }
      }
     
      if (key !== 0)
      {         
         if (my.vt100)
         {
            sendKeyPair(0, evt.charCode, evt);
         }
         else
         {
            sendKey(key, evt);               
         }
      }         
   };
              
   /**
    * Key up event.
    * 
    * @param {event} evt  event object.
    */
   my.onkeyup = function(evt)
   {
      if (evt.keyCode === keys.ALT && evt.location === evt.DOM_KEY_LOCATION_RIGHT)
      {
         altChar = false;
      }
      // stop propagation
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };

   /**
    * Key down event.
    * 
    * @param {event} evt  event object.
    */
   my.onkeydown = function(evt)
   {
      var key = evt.keyCode;
    
      if(altChar || key === keys.CTRL  || key === keys.SHIFT)
      {
         return;
      }

      if (key === keys.ALT)
      {
         if (evt.location === evt.DOM_KEY_LOCATION_RIGHT)
         {
            altChar = evt.ctrlKey;                  
         }
         return;
      }
      
      // In VT100 mode process as key pair. 
      if (my.vt100)
      {
         doVT100(key, evt);
         return;
      }
      
      if (key === keys.ESCAPE)
      {
         evt.preventDefault();
         // ESC is used to clear selection
         if (tty.clipboard.haveSelection())
            tty.clipboard.clearSelection();
         else
            escape = !escape;
         return;               
      }
                               
      var p4code = -1;
      var charCode = tty.keymap.mapKey(evt);
                                       
      // Handle ALT or Escape
      if (evt.altKey || escape)
      {
         p4code = tty.keymap.xlateAlt[key];
         
         if (p4code === undefined)
         {
            if (charCode !== -1 && ((!escape && evt.altKey) || (escape && !evt.altKey)))
            {
               p4code = tty.keymap.processCtrl(evt, charCode);
               if(p4code === -1)
               {
                  p4code = charCode;
               }
               p4code |= 1024;
            }
            else
            {
               p4code = -1;
            }
         }
         else
         {
            if (!escape && evt.altKey && key === keys.DELETE)
            {
               // special case for ALT+DELETE key combo
               p4code = 512;
            }     
         }    
                    
         escape = false;
      }
      // Normal
      else
      {
         /* normal map */
         var p4code = tty.keymap.xlate[key];
         
         if (p4code === undefined)
         {
            // check CTRL
            p4code = tty.keymap.processCtrl(evt, charCode);
         }
         else
         {
            if (p4code === keys.TAB && evt.shiftKey)
            {
               p4code = 509;
            }                           
         }
      }                                                           

      if (p4code !== -1)
      {
         sendKey(p4code, evt);
      }      
   };
               
   /**
    * Process key strokes on VT100 mode.
    * 
    * @param {number} key key code
    * @param {event}  evt event object
    */
   var doVT100 = function(key, evt)
   {
      if (evt.ctrlKey)
      {
         // CTRL keys
         var charCode = tty.keymap.mapKey(evt);
         var ctrlCode = charCode & 0xffbf;
         sendKeyPair(0, ctrlCode, evt);
      }
      else if (evt.altKey)
      {
         // ALT keys are not used on VT100
         evt.preventDefault();
         evt.stopImmediatePropagation();
      }
      else
      {
         switch (key)
         {  
            case keys.TAB:
            case keys.ENTER:
            case keys.ESCAPE:
               sendKeyPair(0, key, evt);
               break;
            case keys.DELETE:
               sendKeyPair(0x7f, 0, evt);
               break;
            case keys.BACKSPACE:
            case keys.PAGE_UP:
            case keys.PAGE_DOWN:
            case keys.LEFT_ARROW:
            case keys.RIGHT_ARROW:
            case keys.UP_ARROW:
            case keys.DOWN_ARROW:
            case keys.INSERT:
            case keys.HOME:
            case keys.END:
            case keys.F1:
            case keys.F2:
            case keys.F3:
            case keys.F4:
            case keys.F5:
            case keys.F6:
            case keys.F7:
            case keys.F8:
            case keys.F9:
            case keys.F10:
            case keys.F11:
            case keys.F12:
               sendKeyPair(key, 0, evt);
               break;
         }
      }
   };
                                                                              
   /**
    * Send key.
    * 
    * @param {number} key key code
    * @param {event}  evt event object
    */
   var sendKey = function(key, evt)
   {
      // CTRL-C (copy) only when have selection
      if (key === 3 && tty.clipboard.haveSelection())
      {
         // Prepare clipboard operations on CTRL key down.
         tty.clipboard.prepareCopy();
         return;
      }

      // CTRL-V (paste)
      if (key === 22)
      {
         tty.clipboard.ctrlv = true;
         if (!window.Clipboard)
         {
            tty.clipboard.preparePaste();
         }
         return;
      }
      
      var data = new Uint8Array(3);
      data[0] = 0x01;
      var c = key;
      data[2] = c & 0xff;
      c = c >> 8;
      data[1] = c & 0xff;
      tty.socket.send(data);      
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };

   /**
    * Send key code and character code.
    * 
    * @param {number} keyCode key code
    * @param {number} charCode key character code
    * @param {event}  evt event object
    */
   var sendKeyPair = function(keyCode, charCode, evt)
   {
      var data = new Uint8Array(5);
      data[0] = 0x02;
      var c = keyCode;
      data[2] = c & 0xff;
      c = c >> 8;
      data[1] = c & 0xff;
      c = charCode;
      data[4] = c & 0xff;
      c = c >> 8;
      data[3] = c & 0xff;
      tty.socket.send(data);
      evt.preventDefault();
      evt.stopImmediatePropagation();
   };
   
   /* exports */
   return my;    
}());

